import { FontAwesome } from "@expo/vector-icons";
import ReactNativeZoomableView from '@openspacelabs/react-native-zoomable-view/src/ReactNativeZoomableView';
import React, { useEffect, useState } from 'react';
import { ActivityIndicator, Alert, Dimensions, GestureResponderEvent, ImageSourcePropType, StatusBar, StyleSheet, TouchableOpacity, View, Text } from 'react-native';
import Picker from 'react-native-picker-select';
import Toast from 'react-native-root-toast';
import { Image, Svg, Text as SvgText } from 'react-native-svg';
import { UseMutationResult } from "react-query";
import { Landmark } from '../../../data/landmarks';
import { MapStackNavigationProp } from "../../../navigation/MapNavigator";
import { colors, lmTypesIndoor } from "../../../utils/GlobalUtils";
import IndoorFloor from '../IndoorFloor';
import ArrowButton from './ArrowButton';
import mapStyles from "./Map.styles";


interface IndoorMapProps {
	navigation: MapStackNavigationProp
	landmarks: Landmark[]
	promptAddLandmark: (longitude?: number, latitude?: number, floor?: number, lmCount?: string, parent?: string) => void
	focusLandmark: (landmark: Landmark) => void
	applyFilter: (landmarks: Landmark[]) => Landmark[]
	togglePreviewPin: (state: boolean) => void
	showPreviewPin: boolean
	selectedLandmarkId: string
	editLmLoc: boolean
	toggleEditLmLoc: (state: boolean) => void
	toggleLmDetails: (state: boolean) => void
	editLandmarkMutation: UseMutationResult<any, unknown, Landmark, unknown>
	updatedLandmark: Landmark,
	setUpdatedLandmark: (landmark: Landmark) => void
}


const IndoorMap: React.FC<IndoorMapProps> = ({ landmarks, promptAddLandmark, focusLandmark, applyFilter, togglePreviewPin, showPreviewPin, selectedLandmarkId, editLmLoc, toggleEditLmLoc, toggleLmDetails, editLandmarkMutation, updatedLandmark, setUpdatedLandmark }) => {
	const [floor, setFloor] = useState(1);
	const [showME, setShowME] = useState(false);
	const [SVGdim, setSVGdim] = useState([1, 1])
	const [localLandmarks, setLocalLandmarks] = useState<Landmark[]>([])
	const imageDim = 0.05 * Dimensions.get("window").width;

	// Local state that tracks active editing coordinates every frame instead of continually updating the actual landmark, then when the edited landmark or preview pin is released, update the actual landmark with this state.
	const [newCoordinates, setNewCoordinates] = useState({ latitude: 0, longitude: 0 })
	const [activeLandmark, setActiveLandmark] = useState<String>("")  // Local state to track active/selected landmark, used to highlight the selected landmark because the timing doesn't work with selectedLandmarkId
	const [lmOpacity, setLmOpacity] = useState<string>("1")

	const editLandmark = async () => {
		if (updatedLandmark) {
			updatedLandmark['editingLocation'] = true  // Tells the back-end we are only editing the landmark's location, nothing else
			toggleEditLmLoc(false)
			toggleLmDetails(true)
			await editLandmarkMutation.mutateAsync(updatedLandmark)
		}
	}

	/**
	 * When landmark location editing is started, find the selected landmark and set its coordinates as the active editing coordinates
	 */
	useEffect(() => {
		if (editLmLoc) {
			applyFilter(landmarks)?.map((item) => {
				if (item.id == selectedLandmarkId) {
					setNewCoordinates({ latitude: item.latitude, longitude: item.longitude })
				}
			})
		}
	}, [editLmLoc])


	useEffect(() => {
		if (editLmLoc) {
			setLmOpacity("0.3")
		}
		else {
			setLmOpacity("1")
		}
	}, [editLmLoc])
    
    /**
     * When there is no longer a landmark selected, revert the opacity of all landmarks to 1.
     */
	useEffect(() => {
		if (selectedLandmarkId == "") {
			setLmOpacity("1")
		}
	}, [selectedLandmarkId])

	// Method to allow users to pick the precise location of their new landmark.
	const chooseLocation = async (e: GestureResponderEvent) => {
		if (!showPreviewPin) {
			console.log('opening preview pin')
			setNewCoordinates({ latitude: (e.nativeEvent.locationY - imageDim / 2) / SVGdim[1], longitude: (e.nativeEvent.locationX - imageDim / 2) / SVGdim[0] })
			togglePreviewPin(true)
		}
	}

	const loadLandmarks = applyFilter(landmarks)?.map((item) => {
		if (!lmTypesIndoor[item.landmark_type]) {
			return null
		}
		if (editLmLoc && item.id == selectedLandmarkId) {  // Landmark's location is currently being edited
			return (
				<View style={{ position: "absolute" }}>
					<Svg>
						<Image
							key={item.id}
							onStartShouldSetResponder={() => true}
							onResponderMove={(e) => { setNewCoordinates({ latitude: (e.nativeEvent.locationY - imageDim / 2) / SVGdim[1], longitude: (e.nativeEvent.locationX - imageDim / 2) / SVGdim[0] }) }}
							onResponderRelease={() => setUpdatedLandmark({ ...item, longitude: newCoordinates.longitude, latitude: newCoordinates.latitude })}
							x={newCoordinates.longitude * SVGdim[0]}
							y={newCoordinates.latitude * SVGdim[1]}
							width={imageDim}
							height={imageDim}
							href={lmTypesIndoor[item.landmark_type].image as ImageSourcePropType} />
						{item.landmark_type == 30 && <SvgText
							x={item.longitude * SVGdim[0] + imageDim * .4}
							y={item.latitude * SVGdim[1] + imageDim * .5}
							fontFamily="sans-serif-medium"
							fontWeight="bold"
							fontSize="8"
							fill="black"
						>{item.groupCount}</SvgText>}
					</Svg>
				</View>
			)
		}
		if (item.floor == floor && item.groupCount != -1 && SVGdim[0] != 1 && SVGdim[1] != 1) {  // Regular indoor landmark
			return (
				<View style={{ position: "absolute" }}>
					<Svg>
						<Image
							opacity={item.id == activeLandmark ? "1" : lmOpacity}
							onPress={() => {
								if (!editLmLoc) {
									setActiveLandmark(item.id)
									setLmOpacity("0.3")
									focusLandmark(item)
								}
							}}
							key={item.id}
							x={item.longitude * SVGdim[0]}
							y={item.latitude * SVGdim[1]}
							width={imageDim}
							height={imageDim}
							href={lmTypesIndoor[item.landmark_type].image as ImageSourcePropType} />
						{item.landmark_type == 30 && <SvgText
							opacity={item.id == activeLandmark ? "1" : lmOpacity}
							x={item.longitude * SVGdim[0] + imageDim * .4}
							y={item.latitude * SVGdim[1] + imageDim * .5}
							fontFamily="sans-serif-medium"
							fontWeight="bold"
							fontSize="8"
							fill="black"
						>{item.groupCount}</SvgText>}
					</Svg>
				</View>
			)
		}
	})


	async function addLandmark(evt: GestureResponderEvent) {
		if (evt != null) {
			if (!showPreviewPin && !editLmLoc) {
				Alert.alert("Are you sure you want to add a landmark here?", undefined,
					[{ text: "Cancel" },
					{
						text: "Confirm", onPress: async () => {
							try {
								chooseLocation(evt)
								// promptAddLandmark((evt.nativeEvent.locationX - imageDim / 2) / SVGdim[0], (evt.nativeEvent.locationY - imageDim / 2) / SVGdim[1], floor)
							}
							catch (err) {
								console.error(err)
								Toast.show("Please ensure finger is not moving when holding down on screen.", { duration: Toast.durations.LONG, })
							}
						}
					}]
				)
			}
		}
	}


	useEffect(() => {
		// Alert.alert("useEffect has been triggered")
		setTimeout(() => setShowME(true), 50);
	})


	const childToWeb = (child: any) => {
		const { type, props } = child;
		const name = type && type.displayName;
		const webName = name && name[0].toLowerCase() + name.slice(1);
		const Tag = webName ? webName : type;
		return <Tag {...props}>{toWeb(props.children)}</Tag>;
	};


	const toWeb = (children: any) => React.Children.map(children, childToWeb);
	function changer(num) {
		setFloor(prevState => prevState + num)
	}


	return (
		<View style={{ height: '100%', width: Dimensions.get("screen").width, backgroundColor: colors.red }}>
			<StatusBar backgroundColor={colors.red} />
			{/* <CustomModal /> */}
			{/* <Text style={{ fontSize: 16, marginBottom: 5 }}>Please select a floor you would like to go to.</Text> */}
			<View style={{ borderColor: "blue", borderWidth: 0, maxHeight: 50, flex: 1, flexDirection: "row", justifyContent: "center", }}>
				{floor == 0 ? <ArrowButton num={0} fontAweIcon={""} /> : <ArrowButton num={-1} fontAweIcon={"chevron-left"} propEvent={() => changer(-1)} />}
				<View style={{ flex: 5, height: 53.5, width: 200 }}>
					<Picker
						placeholder={{}}
						value={floor}
						style={{
							inputIOSContainer: { width: '70%', justifyContent: 'center', alignSelf: 'center' },
							inputAndroid: { textAlign: 'center', color: "white", },
							inputIOS: { color: 'white', textAlign: 'center', height: '100%', alignSelf: 'center' },
							iconContainer: { height: '100%', justifyContent: 'center', }
						}}
						Icon={() => <FontAwesome name="chevron-down" color='white' size={15} />}
						onValueChange={(value) => {
							togglePreviewPin(false)
							setFloor(value)
							setShowME(false)
						}
						}
						items={[
							{ label: 'Basement', value: 0 },
							{ label: 'First Floor', value: 1 },
							{ label: 'Second Floor', value: 2 },
							{ label: 'Third Floor', value: 3 },
							{ label: 'Fourth Floor', value: 4 },
							// { label: 'Fifth Floor', value: 5 }, // Uncomment this to enable the fifth floor
						]}
					/>
				</View>
				{/* {floor == 5 ? arrowBut(0, "") : arrowBut(1, "chevron-right")} */}
				{floor == 4 ? <ArrowButton num={0} fontAweIcon={""} /> : <ArrowButton num={1} fontAweIcon={"chevron-right"} propEvent={() => changer(1)} />}
			</View>
			<View style={{ flex: 1, alignItems: "center", height: '100%', width: '100%', borderColor: 'purple', borderWidth: 0 }}>
				<View style={styles.container}>
					{showME === false ?
						<View style={{ display: 'flex', flexDirection: 'row', justifyContent: "center", }}>
							<ActivityIndicator color={colors.red} size="large" />
						</View> :
						<ReactNativeZoomableView
							panBoundaryPadding={100}
							// bindToBorders={false}
							bindToBorders={true}
							zoomStep={2.8}
							// initialZoom={2.2}
							maxZoom={5}
							minZoom={1}
							initialOffsetY={5}
							movementSensibility={3}
							longPressDuration={200}
							onLongPress={async (e) => { e.persist(); addLandmark(e) }}
						>
							<Svg onLayout={event => {
								setSVGdim([event.nativeEvent.layout.width, event.nativeEvent.layout.height])
								const transformedLandmarks = localLandmarks.map(item => {
									return { ...item, coordx: item.longitude * event.nativeEvent.layout.width, coordy: item.latitude * event.nativeEvent.layout.height }
								})
								//console.log("*this is within onLayout* SVGdim values are " + SVGdim[0] + " AND " + SVGdim[1])
								setLocalLandmarks(transformedLandmarks)
							}}>
								<IndoorFloor floorNum={floor} />
								{loadLandmarks}
								{showPreviewPin &&
									<View style={{ position: "absolute" }}>
										<Svg>
											<Image
												onStartShouldSetResponder={() => true}
												onResponderMove={(e) => { setNewCoordinates({ latitude: (e.nativeEvent.locationY - imageDim / 2) / SVGdim[1], longitude: (e.nativeEvent.locationX - imageDim / 2) / SVGdim[0] }) }}
												x={newCoordinates.longitude * SVGdim[0]}
												y={newCoordinates.latitude * SVGdim[1]}
												width={imageDim}
												height={imageDim}
												href={require("../../../../assets/preview.png")} />
										</Svg>
									</View>
								}
							</Svg>
						</ReactNativeZoomableView>
					}
					{showPreviewPin &&
						<View style={[mapStyles.previewButtonContainer]}>
							<TouchableOpacity style={[mapStyles.previewButtons, mapStyles.cancelPreview]} onPress={() => togglePreviewPin(false)}>
								<Text style={{ color: "white" }}>Cancel</Text>
							</TouchableOpacity>
							<TouchableOpacity style={[mapStyles.previewButtons, mapStyles.confirmPreview]} onPress={() => promptAddLandmark(newCoordinates.longitude, newCoordinates.latitude, floor)}>
								<Text>Add landmark here</Text>
							</TouchableOpacity>
						</View>
					}
					{editLmLoc &&
						<View style={[mapStyles.previewButtonContainer]}>
							<TouchableOpacity style={[mapStyles.previewButtons, mapStyles.cancelPreview]} onPress={() => { toggleEditLmLoc(false); toggleLmDetails(true) }}>
								<Text style={{ color: "white" }}>Cancel</Text>
							</TouchableOpacity>
							<TouchableOpacity style={[mapStyles.previewButtons, mapStyles.confirmPreview]} onPress={() => editLandmark()}>
								<Text>Confirm Location</Text>
							</TouchableOpacity>
						</View>
					}
				</View>
			</View>
		</View >
	);
}

const styles = StyleSheet.create({
	container: {
		flex: 1,
		// backgroundColor: "#fff",
		justifyContent: "center",
		borderColor: "black",
		borderWidth: 0,
		marginVertical: 0,
		aspectRatio: 8 / 10,  // (caters to portrait mode) (width is 80% the value of height dimension)
		width: '100%',
		height: '100%',
		maxWidth: "100%",
		maxHeight: "100%",
		backgroundColor: "white",

	},
	image: {
		alignItems: 'center',
		justifyContent: 'center',
	},
	png: {
		borderColor: "red",
		borderWidth: 3
	},
	arrowButton: {
		flex: 1,
		backgroundColor: colors.red,
		height: 53.5,
	},
});


export default IndoorMap;